class InvalidObjectIdException(Exception):
    def __init__(self, oid: str):
        self.message = f"The given id {oid} does not represent a valid bson.ObjectId"

    def __str__(self) -> str:
        return self.message



import asyncio
import bson
from navalmartin_mir_db_utils.dbs.mongodb_session import MongoDBSession
from navalmartin_mir_db_utils.crud.mongodb_crud_utils import ReadEntityCRUDAPI

IMAGES_COLLECTION_TO_READ = 'YOUR_COLLECTION_NAME'


async def query_db(mongodb_session: MongoDBSession, criteria: dict,
                   projection: dict,
                   collection_name: str):
    query_result = ReadEntityCRUDAPI.do_find(criteria=criteria, projection=projection,
                                             db_session=mongodb_session,
                                             collection_name=collection_name)
    images = [img async for img in query_result]
    return images


async def count_docs(mongodb_session: MongoDBSession, criteria: dict,
                     collection_name: str):
    query_result = await ReadEntityCRUDAPI.count_documents(criteria=criteria,
                                                           db_session=mongodb_session,
                                                           collection_name=collection_name)
    return query_result


def main():
    MONGODB_URL = "YOUR_MONGODB_URL"
    MONGO_DB_NAME_FROM = "YOUR_MONGODB_NAME"
    mir_db_session_from = MongoDBSession(mongodb_url=MONGODB_URL,
                                         db_name=MONGO_DB_NAME_FROM)

    result = asyncio.run(query_db(mongodb_session=mir_db_session_from,
                                  criteria={'survey_idx': bson.ObjectId('63ad64252c853ee163fc6a63')},
                                  projection={'original_filename': 1},
                                  collection_name=IMAGES_COLLECTION_TO_READ))
    print(result)

    # count how many images the survey has
    n_docs = asyncio.run(count_docs(mongodb_session=mir_db_session_from,
                                    criteria={'survey_idx': bson.ObjectId('63ad64252c853ee163fc6a63')},
                                    collection_name=IMAGES_COLLECTION_TO_READ))
    print(n_docs)


if __name__ == '__main__':
    main()



from typing import Any, List
import psutil
from navalmartin_mir_db_utils.crud.mongodb_crud_utils import (CreateEntityCRUDAPI, UpdateEntityCRUDAPI)
from navalmartin_mir_db_utils.dbs.mongodb_session import MongoDBSession
from navalmartin_mir_db_utils.utils.db_enums import TaskStateTypeEnum
from navalmartin_mir_db_utils.dbs.dbs_utils import is_valid_object_id


async def write_to_task_error_collection(event: dict,
                                         error_msg: str,
                                         task_id: str,
                                         error_type: str,
                                         lambda_arn: str,
                                         lambda_request_id: str,
                                         db_session: MongoDBSession,
                                         collection_name: str) -> Any:
    response = CreateEntityCRUDAPI.do_insert_one(data={"event": event,
                                                       'error_msg': error_msg,
                                                       'task_id': task_id,
                                                       "error_type": error_type,
                                                       "lambda_arn": lambda_arn,
                                                       "lambda_request_id": lambda_request_id},
                                                 db_session=db_session,
                                                 collection_name=collection_name)
    return response


async def finish_task(task_id: str,
                      result: dict,
                      total_time: float,
                      db_session: MongoDBSession,
                      collection_name: str) -> Any:
    oid = is_valid_object_id(oid=task_id)
    virtual_mem_dict = dict(psutil.virtual_memory()._asdict())
    cpu_percentage = psutil.cpu_percent()

    data = {'latency': total_time,
            'cpu_util': cpu_percentage,
            'disk_util': virtual_mem_dict,
            'gpu_util': 0.0,
            'cpu_mem_util': 0.0,
            'gpu_mem_util': 0.0,
            'errors': [],
            'task_state': TaskStateTypeEnum.FINISHED.name.upper(),
            'result': result
            }

    response = await UpdateEntityCRUDAPI.do_update_one(criteria={'_id': oid},
                                                       update_data=data,
                                                       db_session=db_session,
                                                       collection_name=collection_name)
    return response


async def finish_task_with_errors(task_id: str,
                                  errors: List[dict],
                                  total_time: float,
                                  db_session: MongoDBSession,
                                  collection_name: str) -> Any:
    """Finish the given task with errors

    Parameters
    ----------
    task_id: The task is
    errors: The errors list
    total_time: The total time the task lasted
    db_session: The MongoDB session
    collection_name: The collection name to write to

    Returns
    -------

    """
    oid = is_valid_object_id(oid=task_id)
    virtual_mem_dict = dict(psutil.virtual_memory()._asdict())
    cpu_percentage = psutil.cpu_percent()

    data = {'latency': total_time,
            'cpu_util': cpu_percentage,
            'disk_util': virtual_mem_dict,
            'gpu_util': 0.0,
            'cpu_mem_util': 0.0,
            'gpu_mem_util': 0.0,
            'errors': errors,
            'task_state': TaskStateTypeEnum.FINISHED.name.upper()
            }
    response = await UpdateEntityCRUDAPI.do_update_one(criteria={'_id': oid},
                                                       update_data=data,
                                                       db_session=db_session,
                                                       collection_name=collection_name)
    return response



"""module mongodb_session. Create a session for the MongoDB

"""
import os

import motor.motor_asyncio
from typing import Union

from navalmartin_mir_db_utils.dbs.dbs_utils import DB_ERROR


class MongoDBSession(object):
    def __init__(self, mongodb_url: str = "", db_name: str = ""):
        self.mongodb_url: Union[str | None] = mongodb_url
        self.db_name: Union[str | None] = db_name
        self.client = None
        self.db = None

        if self.mongodb_url == "" or self.mongodb_url is None:
            self.mongodb_url = os.getenv("MONGODB_URL")

        if self.mongodb_url is None or self.mongodb_url == "":
            print(f"{DB_ERROR} The MongoDB URL is None or empty. Check your configuration")
            raise ValueError("MongoDB URL is not set")

        if self.db_name == "" or self.db_name is None:
            self.db_name = os.getenv("MONGODB_NAME")

        if self.db_name is None or self.db_name == "":
            print(f"{DB_ERROR} The DB name is None or empty. Check your configuration")
            raise ValueError("DB name is not set")

        self.client = motor.motor_asyncio.AsyncIOMotorClient(self.mongodb_url)
        self.db = self.client[self.db_name]


def get_db_session() -> MongoDBSession:
    """Get an instance of MongoDBSession. The application
    should load the environment variables 'MONGODB_URL' and
    'MONGODB_NAME'

    Returns
    -------

    """
    return MongoDBSession()


def get_db_session_with_config(config: dict) -> MongoDBSession:
    mongodb_url = config['MONGODB_URL']
    db_name = config['MONGODB_NAME']
    return MongoDBSession(mongodb_url=mongodb_url, db_name=db_name)


